@@ -17,9 +17,9 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-__id__        = "$Id: J2SE-access-bridge.py,v 1.12 2006/09/29 19:06:25 lmonsanto Exp $"
-__version__   = "$Revision: 1.12 $"
-__date__      = "$Date: 2006/09/29 19:06:25 $"
+__id__        = "$Id: J2SE-access-bridge.py,v 1.13 2006/10/12 00:33:47 wwalker Exp $"
+__version__   = "$Revision: 1.13 $"
+__date__      = "$Date: 2006/10/12 00:33:47 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -47,6 +47,9 @@ from orca_i18n import _ # for gettext support
 class SpeechGenerator(speechgenerator.SpeechGenerator):
     """Overrides _getSpeechForLabel
     """
+    def __init__(self, script):
+        speechgenerator.SpeechGenerator.__init__(self, script)
+        
     def _getSpeechForLabel(self, obj, already_focused):
         """Get the speech for a label.
 
@@ -94,7 +97,7 @@ class Script(default.Script):
     def getSpeechGenerator(self):
         """Returns the speech generator for this script.
         """
-        return SpeechGenerator()
+        return SpeechGenerator(self)
 
     def consumesKeyboardEvent(self, keyboardEvent):
         """Called when a key is pressed on the keyboard.
