@@ -25,12 +25,15 @@ operations on the Accessible object, such as invoking default actions or
 moving the text caret.
 """
 
-__id__        = "$Id: braille.py,v 1.61 2006/09/28 21:09:37 wwalker Exp $"
-__version__   = "$Revision: 1.61 $"
-__date__      = "$Date: 2006/09/28 21:09:37 $"
+__id__        = "$Id: braille.py,v 1.63 2006/10/13 23:39:03 wwalker Exp $"
+__version__   = "$Revision: 1.63 $"
+__date__      = "$Date: 2006/10/13 23:39:03 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import logging
+log = logging.getLogger("braille")
+
 import signal
 import threading
 
@@ -222,10 +225,11 @@ class Region:
         if not string:
             string = ""
 
+        string = string.decode("UTF-8")
         if string[-1:] == "\n":
             string = string[:-1]
+        self.string = string.encode("UTF-8")
 
-        self.string = string
         self.cursorOffset = cursorOffset
 
     def processCursorKey(self, offset):
@@ -316,7 +320,7 @@ class Text(Region):
         self.label = label
         if self.label:
             string = self.label + " " + string
-            cursorOffset += len(unicode(self.label)) + 1
+            cursorOffset += len(self.label.decode("UTF-8")) + 1
 
         Region.__init__(self, string, cursorOffset)
 
@@ -332,7 +336,7 @@ class Text(Region):
         lineOffset = result[2]
         cursorOffset = caretOffset - lineOffset
         if self.label:
-            cursorOffset += len(unicode(self.label)) + 1
+            cursorOffset += len(self.label.decode("UTF-8")) + 1
 
         if lineOffset != self.lineOffset:
             return False
@@ -350,7 +354,7 @@ class Text(Region):
         scrolled off the display."""
 
         if self.label:
-            offset = offset - len(unicode(self.label)) - 1
+            offset = offset - len(self.label.decode("UTF-8")) - 1
             if offset < 0:
                 return
 
@@ -435,9 +439,9 @@ class Line:
         focusOffset = -1
         for region in self.regions:
             if region == _regionWithFocus:
-                focusOffset = len(unicode(string))
+                focusOffset = len(string.decode("UTF-8"))
             if region.string:
-                # [[[TODO: WDW - HACK: Replace unicode ellipses with "..."
+                # [[[TODO: WDW - HACK: Replace UTF-8 ellipses with "..."
                 # The ultimate solution is to get i18n support into
                 # BrlTTY.]]]
                 #
@@ -462,10 +466,10 @@ class Line:
         pos = 0
         for region in self.regions:
             string = string + region.string
-            if len(unicode(string)) > offset:
+            if len(string.decode("UTF-8")) > offset:
                 break
             else:
-                pos = len(unicode(string))
+                pos = len(string.decode("UTF-8"))
 
         return [region, offset - pos]
 
@@ -628,7 +632,7 @@ def refresh(panToCursor=True, targetCursorCell=0):
     # right of the display if we need to pan right.
     #
     if panToCursor and (cursorOffset >= 0):
-        if len(unicode(string)) <= _displaySize[0]:
+        if len(string.decode("UTF-8")) <= _displaySize[0]:
             _viewport[0] = 0
         elif targetCursorCell:
             _viewport[0] = max(0, cursorOffset - targetCursorCell + 1)
@@ -650,22 +654,27 @@ def refresh(panToCursor=True, targetCursorCell=0):
         cursorCell += 1 # Normalize to 1-based offset
 
     debug.println(debug.LEVEL_INFO, "BRAILLE LINE:  '%s'" % string)
+    log.info("line:'%s'" % string)
 
     debug.println(debug.LEVEL_INFO, "     VISIBLE:  '%s', cursor=%d" \
                   % (string[startPos:endPos], cursorCell))
+    log.info("visible:'%s'" % string[startPos:endPos])
+    log.info("cursor:%d" % cursorCell)
 
-    brl.writeText(cursorCell, string[startPos:endPos])
+    string = string.decode("UTF-8")
+    substring = string[startPos:endPos].encode("UTF-8")
+    brl.writeText(cursorCell, substring)
 
     if settings.enableBrailleMonitor:
         if not monitor:
             monitor = brlmon.BrlMon(_displaySize[0])
             monitor.show_all()
-        monitor.writeText(cursorCell, string[startPos:endPos])
+        monitor.writeText(cursorCell, substring)
     elif monitor:
         monitor.destroy()
 
     beginningIsShowing = startPos == 0
-    endIsShowing = endPos >= len(unicode(string))
+    endIsShowing = endPos >= len(string)
 
 def displayRegions(regionInfo):
     """Displays a list of regions on a single line, setting focus to the
@@ -745,7 +754,7 @@ def panRight(panAmount=0):
         lineNum = _viewport[1]
         newX = _viewport[0] + panAmount
         [string, focusOffset] = _lines[lineNum].getLineInfo()
-        if newX < len(unicode(string)):
+        if newX < len(string.decode("UTF-8")):
             _viewport[0] = newX
 
     return oldX != _viewport[0]
