@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 1330 2006-08-17 16:15:03Z richb $"
-__version__   = "$Revision: 1330 $"
-__date__      = "$Date: 2006-08-17 12:15:03 -0400 (Thu, 17 Aug 2006) $"
+__id__        = "$Id: braillegenerator.py,v 1.59 2006/10/12 00:33:47 wwalker Exp $"
+__version__   = "$Revision: 1.59 $"
+__date__      = "$Date: 2006/10/12 00:33:47 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -47,7 +47,13 @@ class BrailleGenerator:
     primary entry point.  Subclasses can feel free to override/extend
     the brailleGenerators instance field as they see fit."""
 
-    def __init__(self):
+    def __init__(self, script):
+
+        # The script that created us.  This allows us to ask the
+        # script for information if we need it.
+        #
+        self._script = script
+
         self.brailleGenerators = {}
         self.brailleGenerators[rolenames.ROLE_ALERT]               = \
              self._getBrailleRegionsForAlert
@@ -124,7 +130,7 @@ class BrailleGenerator:
         self.brailleGenerators[rolenames.ROLE_TABLE]               = \
              self._getBrailleRegionsForTable
         self.brailleGenerators[rolenames.ROLE_TABLE_CELL]          = \
-             self._getBrailleRegionsForTableCell
+             self._getBrailleRegionsForTableCellRow
         self.brailleGenerators[rolenames.ROLE_TABLE_COLUMN_HEADER] = \
              self._getBrailleRegionsForTableColumnHeader
         self.brailleGenerators[rolenames.ROLE_TABLE_ROW_HEADER]    = \
@@ -1009,7 +1015,7 @@ class BrailleGenerator:
         return self._getDefaultBrailleRegions(obj)
 
     def _getBrailleRegionsForTableCell(self, obj):
-        """Get the braille for a table cell
+        """Get the braille for a single table cell
 
         Arguments:
         - obj: the table
@@ -1040,53 +1046,12 @@ class BrailleGenerator:
                     break
 
         if len(regions) == 0:
-            # Adding in a check here to make sure that the parent is a
-            # valid table. It's possible that the parent could be a
-            # table cell too (see bug #351501).
-            #
-            if settings.readTableCellRow and obj.parent.table:
-                rowRegions = []
-                savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
-                settings.brailleVerbosityLevel = \
-                                             settings.VERBOSITY_LEVEL_BRIEF
-
-                parent = obj.parent
-                row = parent.table.getRowAtIndex(obj.index)
-                column = parent.table.getColumnAtIndex(obj.index)
-
-                # This is an indication of whether we should speak all the
-                # table cells (the user has moved focus up or down a row),
-                # or just the current one (focus has moved left or right in
-                # the same row).
-                #
-                speakAll = True
-                if parent.__dict__.has_key("lastRow") and \
-                    parent.__dict__.has_key("lastColumn"):
-                    speakAll = (parent.lastRow != row) or \
-                           ((row == 0 or row == parent.table.nRows-1) and \
-                            parent.lastColumn == column)
-
-                if speakAll:
-                    focusRowRegion = None
-                    for i in range(0, parent.table.nColumns):
-                        accRow = parent.table.getAccessibleAt(row, i)
-                        cell = atspi.Accessible.makeAccessible(accRow)
-                        [cellRegions, focusRegion] = \
-                            self._getDefaultBrailleRegions(\
-                                util.getRealActiveDescendant(cell))
-                        if len(rowRegions):
-                            rowRegions.append(braille.Region(" "))
-                        rowRegions.append(cellRegions[0])
-                        if i == column:
-                            focusRowRegion = cellRegions[0]
-                    regions = [rowRegions, focusRowRegion]
-                    settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
-                else:
-                    regions = self._getDefaultBrailleRegions(
-                        util.getRealActiveDescendant(obj))
-            else:
-                regions = self._getDefaultBrailleRegions(
-                    util.getRealActiveDescendant(obj))
+            regions = self._getDefaultBrailleRegions(
+                              util.getRealActiveDescendant(obj))
+        else:
+            [cellRegions, focusRegion] = \
+              self._getDefaultBrailleRegions(util.getRealActiveDescendant(obj))
+            regions[0].extend(cellRegions)
 
         # [[[TODO: WDW - HACK attempt to determine if this is a node;
         # if so, describe its state.]]]
@@ -1104,6 +1069,69 @@ class BrailleGenerator:
 
         return regions
 
+    def _getBrailleRegionsForTableCellRow(self, obj):
+        """Get the braille for a table cell row or a single table cell
+        if settings.readTableCellRow is False.
+
+        Arguments:
+        - obj: the table
+
+        Returns a list where the first element is a list of Regions to display
+        and the second element is the Region which should get focus.
+        """
+
+        regions = []
+
+        # Adding in a check here to make sure that the parent is a
+        # valid table. It's possible that the parent could be a
+        # table cell too (see bug #351501).
+        #
+        if settings.readTableCellRow and obj.parent.table:
+            rowRegions = []
+            savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
+            settings.brailleVerbosityLevel = \
+                                         settings.VERBOSITY_LEVEL_BRIEF
+
+            parent = obj.parent
+            row = parent.table.getRowAtIndex(obj.index)
+            column = parent.table.getColumnAtIndex(obj.index)
+
+            # This is an indication of whether we should speak all the
+            # table cells (the user has moved focus up or down a row),
+            # or just the current one (focus has moved left or right in
+            # the same row).
+            #
+            speakAll = True
+            if parent.__dict__.has_key("lastRow") and \
+                parent.__dict__.has_key("lastColumn"):
+                speakAll = (parent.lastRow != row) or \
+                       ((row == 0 or row == parent.table.nRows-1) and \
+                        parent.lastColumn == column)
+
+            if speakAll:
+                focusRowRegion = None
+                for i in range(0, parent.table.nColumns):
+                    accRow = parent.table.getAccessibleAt(row, i)
+                    cell = atspi.Accessible.makeAccessible(accRow)
+                    showing = cell.state.count( \
+                                    atspi.Accessibility.STATE_SHOWING)
+                    if showing:
+                        [cellRegions, focusRegion] = \
+                            self._getBrailleRegionsForTableCell(cell)
+                        if len(rowRegions):
+                            rowRegions.append(braille.Region(" "))
+                        rowRegions.append(cellRegions[0])
+                        if i == column:
+                            focusRowRegion = cellRegions[0]
+                regions = [rowRegions, focusRowRegion]
+                settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
+            else:
+                regions = self._getBrailleRegionsForTableCell(obj)
+        else:
+            regions = self._getBrailleRegionsForTableCell(obj)
+
+        return regions
+
     def _getBrailleRegionsForTableColumnHeader(self, obj):
         """Get the braille for a table column header
 
