@@ -20,9 +20,9 @@
 """Provides a graphical braille display at the top of the screen.
 This is mainly for debugging and for demonstrations."""
 
-__id__        = "$Id: brlmon.py,v 1.13 2006/09/28 17:22:29 wwalker Exp $"
-__version__   = "$Revision: 1.13 $"
-__date__      = "$Date: 2006/09/28 17:22:29 $"
+__id__        = "$Id: brlmon.py,v 1.14 2006/10/05 16:31:03 wwalker Exp $"
+__version__   = "$Revision: 1.14 $"
+__date__      = "$Date: 2006/10/05 16:31:03 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -114,10 +114,10 @@ class BrlMon(gtk.Window):
         # Fill out the cells from the string.
         #
         try:
-            string = unicode(string)
+            string = string.decode("UTF-8")
         except:
             string = ""
-        
+
         for i in range(0, len(string)):
 
             # Handle special chars so they are not interpreted by pango.
