@@ -19,12 +19,15 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py,v 1.52 2006/08/30 12:35:15 wwalker Exp $"
-__version__   = "$Revision: 1.52 $"
-__date__      = "$Date: 2006/08/30 12:35:15 $"
+__id__        = "$Id: gnomespeechfactory.py,v 1.55 2006/10/13 23:39:04 wwalker Exp $"
+__version__   = "$Revision: 1.55 $"
+__date__      = "$Date: 2006/10/13 23:39:04 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import logging
+log = logging.getLogger("speech")
+
 import os
 
 import gobject
@@ -36,6 +39,7 @@ import time
 import bonobo
 
 import atspi
+import chnames
 import debug
 import punctuation_settings
 import settings
@@ -43,7 +47,6 @@ import speech
 import speechserver
 
 from acss import ACSS
-from chnames import chnames
 
 from orca_i18n import _           # for gettext support
 
@@ -454,6 +457,8 @@ class SpeechServer(speechserver.SpeechServer):
                                       "SPEECH OUTPUT: '" \
                                       + self.__sayAll.currentContext.utterance\
                                       + "'")
+                        log.info("'%s'" \
+                                 % self.__sayAll.currentContext.utterance)
                         self.__sayAll.idForCurrentContext = self.__speak(
                             self.__sayAll.currentContext.utterance,
                             acss)
@@ -584,7 +589,7 @@ class SpeechServer(speechserver.SpeechServer):
         """
         i = 0
         for text in list:
-            if len(text) != 0:
+            if len(text):
                 self.speak(text, acss, interrupt and (i == 0))
             i += 1
 
@@ -605,16 +610,18 @@ class SpeechServer(speechserver.SpeechServer):
         Returns a text string with the punctuation symbols adjusted accordingly.
         """
 
-        ## Replace ellipses (both manual and unicode) with "dot dot dot"
+        ## Replace ellipses (both manual and UTF-8) with "dot dot dot"
         ##
         oldText = oldText.replace("...", _(" dot dot dot"), 1)
         oldText = oldText.replace("\342\200\246",  _(" dot dot dot"), 1)
+        oldText = oldText.decode("UTF-8")
 
         style = settings.verbalizePunctuationStyle
         newText = ''
         for i in range(0, len(oldText)):
             try:
-                level, action = punctuation_settings.punctuation[oldText[i]]
+                level, action = \
+                    punctuation_settings.getPunctuationInfo(oldText[i])
 
                 # Special case for punctuation in text like filenames or URL's:
                 #
@@ -632,19 +639,19 @@ class SpeechServer(speechserver.SpeechServer):
                 if (len(oldText) == 1) or isSpecial or (style <= level):
                     if isPrev:
                         newText += " "
-                    newText += chnames[oldText[i]]
+                    newText += chnames.getCharacterName(oldText[i])
                     if (action == punctuation_settings.PUNCTUATION_INSERT) \
                         and not isNext:
-                        newText += oldText[i]
+                        newText += oldText[i].encode("UTF-8")
                     if isNext:
                         newText += " "
                 else:
-                    newText += oldText[i]
+                    newText += oldText[i].encode("UTF-8")
             except:
-                if (len(oldText) == 1) and chnames.has_key(oldText[i]):
-                    newText += chnames[oldText[i].lower()]
+                if (len(oldText) == 1):
+                    newText += chnames.getCharacterName(oldText[i])
                 else:
-                    newText += oldText[i]
+                    newText += oldText[i].encode("UTF-8")
 
         return newText
 
@@ -666,6 +673,11 @@ class SpeechServer(speechserver.SpeechServer):
         be spoken.
         """
 
+        # If the user has speech turned off, just return.
+        #
+        if not settings.enableSpeech:
+            return -1
+
         speaker = self.__getSpeaker(acss)
         if acss and not acss.has_key(ACSS.RATE):
             voices = settings.voices
@@ -736,6 +748,7 @@ class SpeechServer(speechserver.SpeechServer):
             [context, acss] = utteranceIterator.next()
             debug.println(debug.LEVEL_INFO,
                           "SPEECH OUTPUT: '" + context.utterance + "'")
+            log.info("'%s'" % context.utterance)
             self.__sayAll = _SayAll(utteranceIterator,
                                     context,
                                     self.__speak(context.utterance, acss),
