@@ -21,9 +21,9 @@
 """Provides a methods that converts the role name of an Accessible
 object into localized strings for speech and braille."""
 
-__id__        = "$Id: rolenames.py 1163 2006-07-13 13:46:48Z wwalker $"
-__version__   = "$Revision: 1163 $"
-__date__      = "$Date: 2006-07-13 09:46:48 -0400 (Thu, 13 Jul 2006) $"
+__id__        = "$Id: rolenames.py,v 1.27 2006/10/12 18:13:03 wwalker Exp $"
+__version__   = "$Revision: 1.27 $"
+__date__      = "$Date: 2006/10/12 18:13:03 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -57,18 +57,23 @@ ROLE_DESKTOP_FRAME       = "desktop frame"
 ROLE_DIAL                = "dial"
 ROLE_DIALOG              = "dialog"
 ROLE_DIRECTORY_PANE      = "directory pane"
+ROLE_DOCUMENT_FRAME      = "document frame"
 ROLE_DRAWING_AREA        = "drawing area"
+ROLE_ENTRY               = "entry"
 ROLE_FILE_CHOOSER        = "file chooser"
 ROLE_FILLER              = "filler"
 ROLE_FONT_CHOOSER        = "fontchooser"
 ROLE_FRAME               = "frame"
 ROLE_GLASS_PANE          = "glass pane"
+ROLE_HEADING             = "heading"
 ROLE_HTML_CONTAINER      = "html container"
 ROLE_ICON                = "icon"
 ROLE_IMAGE               = "image"
 ROLE_INTERNAL_FRAME      = "internal frame"
+ROLE_INPUT_METHOD_WINDOW = "input method window"
 ROLE_LABEL               = "label"
 ROLE_LAYERED_PANE        = "layered pane"
+ROLE_LINK                = "link"
 ROLE_LIST                = "list"
 ROLE_LIST_ITEM           = "list item"
 ROLE_MENU                = "menu"
@@ -268,6 +273,11 @@ rolenames[ROLE_GLASS_PANE] = Rolename(ROLE_GLASS_PANE,
                                       _("GlassPane"),
                                       _("glass pane"))
 
+rolenames[ROLE_HEADING] = Rolename(ROLE_HEADING,
+                                   _("heading"),
+                                   _("Heading"),
+                                   _("hdng"))
+
 rolenames[ROLE_HTML_CONTAINER] = Rolename(ROLE_HTML_CONTAINER,
                                           _("html"),
                                           _("HtmlContainer"),
@@ -458,6 +468,8 @@ rolenames[ROLE_TEXT] = Rolename(ROLE_TEXT,
                                 _("Text"),
                                 _("text"))
 
+rolenames[ROLE_ENTRY] = rolenames[ROLE_TEXT]
+
 rolenames[ROLE_TOGGLE_BUTTON] = Rolename(ROLE_TOGGLE_BUTTON,
                                          _("tglbtn"),
                                          _("ToggleButton"),
