@@ -19,9 +19,9 @@
 
 """Custom script for Evolution."""
 
-__id__        = "$Id: Evolution.py,v 1.60 2006/06/27 14:25:07 richb Exp $"
-__version__   = "$Revision: 1.60 $"
-__date__      = "$Date: 2006/06/27 14:25:07 $"
+__id__        = "$Id: Evolution.py,v 1.67 2006/09/29 17:02:23 wwalker Exp $"
+__version__   = "$Revision: 1.67 $"
+__date__      = "$Date: 2006/09/29 17:02:23 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -30,8 +30,8 @@ import orca.default as default
 import orca.atspi as atspi
 import orca.input_event as input_event
 import orca.rolenames as rolenames
-import orca.orca as orca
 import orca.braille as braille
+import orca.orca_state as orca_state
 import orca.speech as speech
 import orca.settings as settings
 import orca.util as util
@@ -102,9 +102,10 @@ class Script(default.Script):
 
         default.Script.setupInputEventHandlers(self)
 
-        self.sayAllHandler = input_event.InputEventHandler(
-            Script.sayAll,
-            _("Speaks entire document."))
+        self.inputEventHandlers["sayAllHandler"] = \
+            input_event.InputEventHandler(
+                Script.sayAll,
+                _("Speaks entire document."))
 
     def speakSetupAssistantLabel(self, label):
         """Perform a variety of tests on this Setup Assistant label to see
@@ -131,7 +132,7 @@ class Script(default.Script):
 
             # Each Setup Assistant screen has one label in the top left
             # corner that describes what this screen is for. It has a text
-            # weight attribute of 800. We always speak those labels with 
+            # weight attribute of 800. We always speak those labels with
             # " screen" appended.
             #
             if label.text:
@@ -141,8 +142,27 @@ class Script(default.Script):
                     weight = charDict.get('weight')
                     if weight and weight == '800':
                         text = util.getDisplayedText(label)
-                        if text:
-                            speech.speak(text + _(" screen"))
+
+                        # Only speak the screen label if we haven't already
+                        # done so.
+                        #
+                        if text and not self.setupLabels.has_key(label):
+                            speech.speak(text + _(" screen"), None, False)
+                            self.setupLabels[label] = True
+
+                            # If the locus of focus is a push button that's
+                            # insensitive, speak/braille about it. (The
+                            # Identity screen has such a component).
+                            #
+                            if orca_state.locusOfFocus and \
+                               orca_state.locusOfFocus.role == \
+                                   rolenames.ROLE_PUSH_BUTTON and \
+                               (orca_state.locusOfFocus.state.count( \
+                                   atspi.Accessibility.STATE_SENSITIVE) == 0):
+                                self.updateBraille(orca_state.locusOfFocus)
+                                speech.speakUtterances(
+                                    self.speechGenerator.getSpeech( \
+                                        orca_state.locusOfFocus, False))
 
             # It's possible to get multiple "object:state-changed:showing"
             # events for the same label. If we've already handled this
@@ -159,11 +179,11 @@ class Script(default.Script):
                 if text.startswith(_("Please")) or \
                     text.startswith(_("Welcome")) or \
                     text.startswith(_("Congratulations")):
-                    speech.speak(text)
+                    speech.speak(text, None, False)
                     self.setupLabels[label] = True
 
     def handleSetupAssistantPanel(self, panel):
-        """Find all the labels in this Setup Assistant panel and see if 
+        """Find all the labels in this Setup Assistant panel and see if
         we want to speak them.
 
         Arguments:
@@ -221,10 +241,30 @@ class Script(default.Script):
 
         return hrs + ' ' + mins + ' ' + suffix
 
+    def textLines(self):
+        """This an iterator that produces elements of the form:
+        [SayAllContext, acss], where SayAllContext has the text to be
+        spoken and acss is an ACSS instance for speaking the text.  We
+        do this because of the way Evolution lays out its messages:
+        each paragraph tends to be in its own panel, each of which is
+        in a higher level panel.  So, we just traverse through the
+        children.
+        """
+        panel = orca_state.locusOfFocus.parent
+        htmlPanel = orca_state.locusOfFocus.parent.parent
+        startIndex = panel.index
+        for i in range(startIndex, htmlPanel.childCount):
+            accPanel = htmlPanel.accessible.getChildAtIndex(i)
+            panel = atspi.Accessible.makeAccessible(accPanel)
+            accTextObj = panel.accessible.getChildAtIndex(0)
+            textObj = atspi.Accessible.makeAccessible(accTextObj)
+            for [context, acss] in util.textLines(textObj):
+                yield [context, acss]
+
     def sayAll(self, inputEvent):
         """Speak all the text associated with the text object that has
            focus. We have to define our own method here because Evolution
-           does now implement the FLOWS_TO relationship and all the text
+           does not implement the FLOWS_TO relationship and all the text
            are in an HTML panel which contains multiple panels, each
            containing a single text object.
 
@@ -233,23 +273,11 @@ class Script(default.Script):
         """
 
         debug.println(self.debugLevel, "evolution.sayAll.")
-        if orca.locusOfFocus and orca.locusOfFocus.text:
-
-            # Get the HTML panel containing all the other panels (each
-            # of which contains a text object). Starting at the current
-            # one, get a handle to each text object in turn, and speak it.
-            #
-            panel = orca.locusOfFocus.parent
-            htmlPanel = orca.locusOfFocus.parent.parent
-            startIndex = panel.index
-            for i in range(startIndex, htmlPanel.childCount):
-                accPanel = htmlPanel.accessible.getChildAtIndex(i)
-                panel = atspi.Accessible.makeAccessible(accPanel)
-                accTextObj = panel.accessible.getChildAtIndex(0)
-                textObj = atspi.Accessible.makeAccessible(accTextObj)
-
-                speech.sayAll(util.textLines(textObj),
-                              self.__sayAllProgressCallback)
+        if orca_state.locusOfFocus and orca_state.locusOfFocus.text:
+            #for [context, acss] in self.textLines():
+            #    print context.utterance
+            speech.sayAll(self.textLines(),
+                          self.__sayAllProgressCallback)
         else:
             default.Script.sayAll(self, inputEvent)
 
@@ -312,7 +340,8 @@ class Script(default.Script):
             braille.displayMessage(result[0])
             if settings.enableSpeechIndentation:
                 self.speakTextIndentation(event.source, result[0])
-            speech.speak(result[0])
+            line = util.adjustForRepeats(result[0])
+            speech.speak(line)
             return
 
         # 2) Mail view: current message pane: "standard" mail header lines.
@@ -436,7 +465,7 @@ class Script(default.Script):
             # setting of the speakAll variable above, incorrect. We fix that
             # up here.
 
-            if orca.locusOfFocus.role != rolenames.ROLE_TABLE_CELL:
+            if orca_state.locusOfFocus.role != rolenames.ROLE_TABLE_CELL:
                 speakAll = True
                 message = "%d messages" % \
                     parent.table.nRows
@@ -463,7 +492,7 @@ class Script(default.Script):
                                 checkbox = True
                                 checked = cell.state.count( \
                                     atspi.Accessibility.STATE_CHECKED)
-                                if not checked and speakAll == True:
+                                if not checked and speakAll:
                                     toRead = False
                                 break
 
@@ -652,7 +681,7 @@ class Script(default.Script):
                                                 brailleRegions[0]])
                         found = True
 
-            if found == False:
+            if not found:
                 startTime = 'Start time ' + self.getTimeForCalRow(index, noRows)
                 brailleRegions.append(braille.Region(startTime))
                 speech.speak(startTime)
@@ -783,7 +812,7 @@ class Script(default.Script):
 
             # Braille the default action for this component.
             #
-            self.updateBraille(orca.locusOfFocus)
+            self.updateBraille(orca_state.locusOfFocus)
 
             # Look for the "Suggestions for 'xxxxx' label in the spell
             # checker dialog panel. Extract out the xxxxx. This will be the
@@ -815,7 +844,7 @@ class Script(default.Script):
                 panel = self.message_panel
                 allText = util.findByRole(panel, rolenames.ROLE_TEXT)
                 for i in range(0, len(allText)):
-                    text = allText[i].text.getText(0, -1)
+                    text = self.getText(allText[i], 0, -1)
                     tokens = text.split()
                     allTokens += tokens
 
@@ -856,7 +885,7 @@ class Script(default.Script):
             table = event.source.parent.parent.parent.parent.parent
             cell = table.child(table.childCount-1)
             allText = util.findByRole(cell, rolenames.ROLE_TEXT)
-            utterance = "for " + allText[0].text.getText(0, -1)
+            utterance = "for " + self.getText(allText[0], 0, -1)
             speech.speak(utterance)
             return
 
@@ -866,7 +895,7 @@ class Script(default.Script):
         # "Evolution Setup Assistant", then empty out the two dictionaries
         # containing which setup assistant panels and labels we've already
         # seen.
- 
+
         obj = event.source.parent
         while obj and obj.role != rolenames.ROLE_APPLICATION:
             if obj.role == rolenames.ROLE_FRAME and \
@@ -890,7 +919,7 @@ class Script(default.Script):
 
     def onStateChanged(self, event):
         """Called whenever an object's state changes.  We are only
-        interested in "object:state-changed:showing" events for any 
+        interested in "object:state-changed:showing" events for any
         object in the Setup Assistant.
 
         Arguments:
@@ -919,7 +948,7 @@ class Script(default.Script):
                         self.speakSetupAssistantLabel(event.source)
                         break
 
-                    # If the event is for a panel and we haven't already 
+                    # If the event is for a panel and we haven't already
                     # seen this panel, then handle it.
                     #
                     elif event.source.role == rolenames.ROLE_PANEL and \
@@ -927,6 +956,7 @@ class Script(default.Script):
                         self.handleSetupAssistantPanel(event.source)
                         self.setupPanels[event.source] = True
                         break
+
                 obj = obj.parent
 
         # For everything else, pass the event onto the parent class
