@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py,v 1.74 2006/07/21 16:30:47 wwalker Exp $"
-__version__   = "$Revision: 1.74 $"
-__date__      = "$Date: 2006/07/21 16:30:47 $"
+__id__        = "$Id: speechgenerator.py,v 1.79 2006/10/12 00:33:47 wwalker Exp $"
+__version__   = "$Revision: 1.79 $"
+__date__      = "$Date: 2006/10/12 00:33:47 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -33,7 +33,7 @@ import math
 
 import atspi
 import debug
-import orca
+import orca_state
 import rolenames
 import settings
 import util
@@ -46,8 +46,13 @@ class SpeechGenerator:
     entry point.  Subclasses can feel free to override/extend the
     speechGenerators instance field as they see fit."""
 
-    def __init__(self):
+    def __init__(self, script):
 
+        # The script that created us.  This allows us to ask the
+        # script for information if we need it.
+        #
+        self._script = script
+        
         # Set up a dictionary that maps role names to functions
         # that generate speech for objects that implement that role.
         #
@@ -127,7 +132,7 @@ class SpeechGenerator:
         self.speechGenerators[rolenames.ROLE_TABLE]               = \
              self._getSpeechForTable
         self.speechGenerators[rolenames.ROLE_TABLE_CELL]          = \
-             self._getSpeechForTableCell
+             self._getSpeechForTableCellRow
         self.speechGenerators[rolenames.ROLE_TABLE_COLUMN_HEADER] = \
              self._getSpeechForTableColumnHeader
         self.speechGenerators[rolenames.ROLE_TABLE_ROW_HEADER]    = \
@@ -688,7 +693,7 @@ class SpeechGenerator:
 
         utterances = self._getDefaultSpeech(obj, already_focused)
 
-        if (obj == orca.locusOfFocus) \
+        if (obj == orca_state.locusOfFocus) \
                and (settings.speechVerbosityLevel \
                     == settings.VERBOSITY_LEVEL_VERBOSE):
             utterances.extend(self._getSpeechForObjectAccelerator(obj))
@@ -1130,7 +1135,7 @@ class SpeechGenerator:
         return utterances
 
     def _getSpeechForTableCell(self, obj, already_focused):
-        """Get the speech for a table cell
+        """Get the speech utterances for a single table cell
 
         Arguments:
         - obj: the table
@@ -1150,7 +1155,7 @@ class SpeechGenerator:
         if action:
             for i in range(0, action.nActions):
                 debug.println(debug.LEVEL_FINEST,
-                    "speechgenerator._getSpeechForTableCell " \
+                    "speechgenerator.__getTableCellUtterances " \
                     + "looking at action %d" % i)
                 if action.getName(i) == "toggle":
                     obj.role = rolenames.ROLE_CHECK_BOX
@@ -1159,7 +1164,39 @@ class SpeechGenerator:
                     obj.role = rolenames.ROLE_TABLE_CELL
                     break
 
-        if (len(utterances) == 0) and (not already_focused):
+        utterances.append(util.getDisplayedText(\
+                          util.getRealActiveDescendant(obj)))
+
+        # [[[TODO: WDW - HACK attempt to determine if this is a node;
+        # if so, describe its state.]]]
+        #
+        if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
+            if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
+                utterances.append(_("expanded"))
+            else:
+                utterances.append(_("collapsed"))
+
+        self._debugGenerator("_getSpeechForTableCell",
+                             obj,
+                             already_focused,
+                             utterances)
+
+        return utterances
+
+    def _getSpeechForTableCellRow(self, obj, already_focused):
+        """Get the speech for a table cell row or a single table cell
+        if settings.readTableCellRow is False.
+
+        Arguments:
+        - obj: the table
+        - already_focused: False if object just received focus
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = []
+
+        if (not already_focused):
             if settings.readTableCellRow:
                 parent = obj.parent
                 row = parent.table.getRowAtIndex(obj.index)
@@ -1177,32 +1214,20 @@ class SpeechGenerator:
                            ((row == 0 or row == parent.table.nRows-1) and \
                             parent.lastColumn == column)
 
-                if speakAll == True:
+                if speakAll:
                     for i in range(0, parent.table.nColumns):
                         accRow = parent.table.getAccessibleAt(row, i)
                         cell = atspi.Accessible.makeAccessible(accRow)
-                        utterances.append(util.getDisplayedText(\
-                            util.getRealActiveDescendant(cell)))
+                        showing = cell.state.count( \
+                                          atspi.Accessibility.STATE_SHOWING)
+                        if showing:
+                            utterances.extend(self._getSpeechForTableCell(cell, 
+                                                           already_focused))
                 else:
-                    utterances.append(util.getDisplayedText(\
-                    util.getRealActiveDescendant(obj)))
+                    utterances.extend(self._getSpeechForTableCell(obj,
+                                                           already_focused))
             else:
-                utterances.append(util.getDisplayedText(\
-                    util.getRealActiveDescendant(obj)))
-
-        # [[[TODO: WDW - HACK attempt to determine if this is a node;
-        # if so, describe its state.]]]
-        #
-        if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
-            if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
-                utterances.append(_("expanded"))
-            else:
-                utterances.append(_("collapsed"))
-
-        self._debugGenerator("_getSpeechForTableCell",
-                             obj,
-                             already_focused,
-                             utterances)
+                utterances = self._getSpeechForTableCell(obj, already_focused)
 
         return utterances
 
@@ -1470,6 +1495,7 @@ class SpeechGenerator:
             if parent == stopAncestor:
                 break
             if (parent.role != rolenames.ROLE_FILLER) \
+                and (parent.role != rolenames.ROLE_LAYERED_PANE) \
                 and (parent.role != rolenames.ROLE_SPLIT_PANE) \
                 and (parent.role != rolenames.ROLE_SCROLL_PANE) \
                 and (parent.role != rolenames.ROLE_UNKNOWN):
@@ -1485,7 +1511,7 @@ class SpeechGenerator:
                    and (((not label) or (len(label) == 0) \
                          or (not text) or (len(text) == 0))):
                     pass
-                else:
+                elif parent.role != rolenames.ROLE_TABLE_CELL:
                     utterances.append(rolenames.getSpeechForRoleName(parent))
 
                 if text and len(text):
@@ -1493,6 +1519,9 @@ class SpeechGenerator:
                 if label and len(label):
                     utterances.append(label)
 
+                if parent.role == rolenames.ROLE_TABLE_CELL:
+                    utterances.append(rolenames.getSpeechForRoleName(parent))
+
             # [[[TODO: HACK - we've discovered oddness in hierarchies
             # such as the gedit Edit->Preferences dialog.  In this
             # dialog, we have labeled groupings of objects.  The
